@@ -1,6 +1,6 @@
 """A clean customisable Sphinx documentation theme."""
 
-__version__ = "2023.09.10"
+__version__ = "2024.07.18"
 
 import hashlib
 import logging
@@ -14,6 +14,7 @@ from docutils import nodes
 from pygments.formatters import HtmlFormatter
 from pygments.style import Style
 from pygments.token import Text
+from sphinx.builders.dirhtml import DirectoryHTMLBuilder
 from sphinx.builders.html import StandaloneHTMLBuilder
 from sphinx.environment.adapters.toctree import TocTree
 from sphinx.errors import ConfigError
@@ -75,8 +76,8 @@ def has_not_enough_items_to_show_toc(
     except IndexError:
         val = True
     else:
-        # There's only the page's own toctree in there.
-        val = len(self_toctree) == 1 and self_toctree[0].tagname == "toctree"
+        # There's only the page's own toctree(s) in there.
+        val = all(entry.tagname == "toctree" for entry in self_toctree)
     return val
 
 
@@ -178,6 +179,27 @@ def _add_asset_hashes(static: List[str], add_digest_to: List[str]) -> None:
         static[index].filename = _asset_hash(asset)  # type: ignore[attr-defined]
 
 
+def _fix_canonical_url(
+    app: sphinx.application.Sphinx, pagename: str, context: Dict[str, Any]
+) -> None:
+    """Fix the canonical URL when using the dirhtml builder.
+
+    Sphinx builds a canonical URL if ``html_baseurl`` config is set. However,
+    it builds a URL ending with ".html" when using the dirhtml builder, which is
+    incorrect. Detect this and generate the correct URL for each page.
+    """
+    if (
+        not app.config.html_baseurl
+        or not isinstance(app.builder, DirectoryHTMLBuilder)
+        or not context["pageurl"]
+        or not context["pageurl"].endswith(".html")
+    ):
+        return
+
+    target = app.builder.get_target_uri(pagename)
+    context["pageurl"] = app.config.html_baseurl + target
+
+
 def _html_page_context(
     app: sphinx.application.Sphinx,
     pagename: str,
@@ -196,6 +218,8 @@ def _html_page_context(
             ["scripts/furo.js"],
         )
 
+    _fix_canonical_url(app, pagename, context)
+
     # Basic constants
     context["furo_version"] = __version__
 
@@ -259,7 +283,13 @@ def _builder_inited(app: sphinx.application.Sphinx) -> None:
     update_known_styles_state(app)
 
     def _update_default(key: str, *, new_default: Any) -> None:
-        app.config.values[key] = (new_default, *app.config.values[key][1:])
+        try:
+            conf_py_settings = app.config._raw_config
+        except AttributeError:
+            pass  # Sphinx's config has changed.
+        else:
+            if key not in conf_py_settings:
+                app.config._raw_config.setdefault(key, new_default)
 
     # Change the default permalinks icon
     _update_default("html_permalinks_icon", new_default="#")
@@ -278,8 +308,8 @@ def update_known_styles_state(app: sphinx.application.Sphinx) -> None:
 def _get_light_style(app: sphinx.application.Sphinx) -> Style:
     # fmt: off
     # For https://github.com/psf/black/issues/3869
-    return (
-        app  # type: ignore[no-any-return]
+    return (  # type: ignore[no-any-return]
+        app
             .builder
             .highlighter # type: ignore[attr-defined]
             .formatter_args["style"]
